<?php

App::uses('AppHelper', 'View/Helper');
App::uses('BasicPermissions', 'EvCore.Lib');

class PermissionsHelper extends AppHelper {

	public $helpers = [
		'Form'
	];

	public function __construct(View $View, $settings = array()) {
		parent::__construct($View, $settings);

		$this->Permissions = new BasicPermissions(
			EvClassRegistry::init('EvCore.UserGroup')
		);
	}

	/**
	 * magic method to check if the function exists in the lib file and call it if so
	 * if not, it will try and return the string we were trying to Inflect
	 * if it can't do that, it will just return false
	 *
	 */
	public function __call($name, $arguments) {
		if (method_exists($this->Permissions, $name)) {

			return call_user_func_array(array($this->Permissions, $name), $arguments);
		}

		// we can't find it, return false
		return false;
	}

	public function injectAdminForm() {
		if (!empty($this->_View->viewVars['permissions'])) {
			echo $this->element(
				'form_tab',
				array(
					'title' => 'Permissions',
					'tabContent' => $this->element('EvCore.UserGroups/permissions', compact('permissions')),
					'divOptions' => [
						'class' => 'user-group-permissions'
					]
				)
			);
		}
	}

	public function adminDisplayPermissions($permissions, $modelName, $modelId, $depth = 0, $prefix = '') {
		$content = '';

		foreach ($permissions as $permission) {
			$permissionChildren = $permission['children'];

			if (!empty($permissionChildren) && $depth === 0) {
				//Top level category and has children
				$fieldsetClass = 'permission top-level-permission has-children collapsable-fieldset col-xs-12';
				$legendClass = 'col-xs-3';
			} elseif (empty($permissionChildren) && $depth === 0) {
				//Top level category and no children
				$fieldsetClass = 'permission top-level-permission no-children collapsable-fieldset col-xs-12';
				$legendClass = '';
			} elseif (!empty($permissionChildren)) {
				//category with children
				$fieldsetClass = 'permission has-children collapsable-fieldset col-xs-12';
				$legendClass = '';
			} else {
				//category without children
				$fieldsetClass = 'permission col-xs-12';
				$legendClass = '';
			}

			$content .= '<fieldset class="' . $fieldsetClass . '" data-depth="' . $depth . '"">';

			$content .= '<legend class="' . $legendClass . '">';

			$permissionPermissions = Hash::extract($permission['Aro'], '{n}[model=' . $modelName . '][foreign_key=' . $modelId . ']');
			$allowChecked = false;
			$denyChecked = false;
			if (!empty($permissionPermissions[0]['Permission'])) {
				if ($this->isPermissionAllowed($permissionPermissions[0]['Permission'])) {
					$allowChecked = true;
				} else {
					$denyChecked = true;
				}
			}

			$checkboxDivClass = (!empty($permissionChildren) || $depth === 0 ) ? 'col-xs-11' : 'col-xs-11';
			$content .= $this->_adminDisplayPermissionCheckboxes($permission['Aco']['alias'], $permission['Aco']['id'], $checkboxDivClass, $allowChecked, $denyChecked);

			$content .= '</legend>';

			if (!empty($permissionChildren)) {
				$content .= '<div>';
				$depth++;
				$content .= $this->adminDisplayPermissions($permissionChildren, $modelName, $modelId, $depth, $prefix . '/' . $permission['Aco']['alias']);
				$depth--;
				$content .= '</div>';
			}

			$content .= '</fieldset>';
		}

		return $content;
	}

	protected function _adminDisplayPermissionCheckboxes($permissionName, $permissionId, $divClass = 'col-xs-11', $allowChecked = false, $denyChecked = false) {
		$content = '<div class="checkbox ' . $divClass . '" style="margin-left:25px;padding-top:0;">';

		$content .= $this->Form->checkbox('Permission.' . $permissionName . '.allow', array(
			'type' => 'checkbox',
			'value' => $permissionId,
			'hiddenField' => false,
			'checked' => $allowChecked,
			'class' => 'col-xs-1',
		));

		$content .= $this->Form->label(
			'Permission.' . $permissionName . '.allow',
			'Allow ' . $permissionName,
			[
				'class' => 'col-xs-4',
			]
		);

		$content .= $this->Form->checkbox('Permission.' . $permissionName . '.deny', array(
			'type' => 'checkbox',
			'value' => $permissionId,
			'hiddenField' => false,
			'checked' => $denyChecked,
			'class' => 'col-xs-1',
		));

		$content .= $this->Form->label(
			'Permission.' . $permissionName . '.deny',
			'Deny ' . $permissionName,
			[
				'class' => 'col-xs-4',
			]
		);

		$content .= '</div>';

		return $content;
	}

	public function isPermissionAllowed($permission, $type = null) {
		if (!empty($type)) {
			if ($permission[$type] == 1) {
				return true;
			}
		} else {
			foreach (array_keys($permission) as $permissionType) {
				if (strpos($permissionType, '_') === 0) {
					if ($permission[$permissionType] != 1) {
						return false;
					}
				}
			}

			return true;
		}

		return null;
	}
}
